@@ -1,4 +1,4 @@
-#![cfg_attr(docsrs, feature(doc_auto_cfg, doc_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 #![doc = include_str!("../README.md")]
 // @@ begin lint list maintained by maint/add_warning @@
 #![allow(renamed_and_removed_lints)] // @@REMOVE_WHEN(ci_arti_stable)
@@ -41,11 +41,15 @@
 #![allow(clippy::result_large_err)] // temporary workaround for arti#587
 #![allow(clippy::needless_raw_string_hashes)] // complained-about code is fine, often best
 #![allow(clippy::needless_lifetimes)] // See arti#1765
+#![allow(mismatched_lifetime_syntaxes)] // temporary workaround for arti#2060
 //! <!-- @@ end lint list maintained by maint/add_warning @@ -->
 
 // TODO #1645 (either remove this, or decide to have it everywhere)
 #![cfg_attr(not(all(feature = "full", feature = "experimental")), allow(unused))]
 
+#[cfg(feature = "parse2")]
+#[macro_use]
+pub mod parse2;
 #[cfg(feature = "hs-service")]
 pub(crate) mod build;
 #[macro_use]
@@ -55,6 +59,9 @@ mod err;
 pub mod types;
 mod util;
 
+#[doc(hidden)]
+pub use derive_deftly;
+
 // Use `#[doc(hidden)]` rather than pub(crate), because otherwise the doctest
 // doesn't work.
 #[doc(hidden)]
@@ -87,3 +94,47 @@ pub enum AllowAnnotations {
     /// Parsing a document where annotations are not allowed.
     AnnotationsNotAllowed,
 }
+
+/// Return a list of the protocols [supported](tor_protover::doc_supported)
+/// by this crate.
+pub fn supported_protocols() -> tor_protover::Protocols {
+    use tor_protover::named::*;
+    // WARNING: REMOVING ELEMENTS FROM THIS LIST CAN BE DANGEROUS!
+    // SEE [`tor_protover::doc_changing`]
+    [
+        DESC_CROSSSIGN,
+        DESC_NO_TAP,
+        DESC_FAMILY_IDS,
+        MICRODESC_ED25519_KEY,
+        MICRODESC_NO_TAP,
+        CONS_ED25519_MDS,
+    ]
+    .into_iter()
+    .collect()
+}
+
+#[cfg(test)]
+mod test {
+    // @@ begin test lint list maintained by maint/add_warning @@
+    #![allow(clippy::bool_assert_comparison)]
+    #![allow(clippy::clone_on_copy)]
+    #![allow(clippy::dbg_macro)]
+    #![allow(clippy::mixed_attributes_style)]
+    #![allow(clippy::print_stderr)]
+    #![allow(clippy::print_stdout)]
+    #![allow(clippy::single_char_pattern)]
+    #![allow(clippy::unwrap_used)]
+    #![allow(clippy::unchecked_duration_subtraction)]
+    #![allow(clippy::useless_vec)]
+    #![allow(clippy::needless_pass_by_value)]
+    //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
+
+    use super::*;
+
+    #[test]
+    fn protocols() {
+        let pr = supported_protocols();
+        let expected = "Cons=2 Desc=2-4 Microdesc=2-3".parse().unwrap();
+        assert_eq!(pr, expected);
+    }
+}
